from google.colab import files
import os
import time,datetime
import numpy as np
import cv2

def main():
	if os.path.exists('aolp.npy'):
		os.remove('aolp.npy')
	if os.path.exists('iavg.npy'):
		os.remove('iavg.npy')
	if os.path.exists('inside.bmp'):
		os.remove('inside.bmp')
	print('Upload aolp.npy')
	print('numpy. 1 channel. 0~pi. start from x-axis. counter clockwise.')
	files.upload()
	print('Upload iavg.npy')
	print('numpy. 1 channel or 3 channel bgr. 0~255.')
	files.upload()
	print('Upload inside.bmp')
	print('uncompressed image. 1 channel. 0:outside, 255:inside.')
	files.upload()

	print('File read...',end='',flush=True)
	aolp=np.load('aolp.npy')
	iavg=np.load('iavg.npy')
	insideimage=cv2.imread('inside.bmp',cv2.IMREAD_GRAYSCALE)
	rows,cols=insideimage.shape
	inside=(insideimage>127)
	inside[0,:]=False
	inside[rows-1,:]=False
	inside[:,0]=False
	inside[:,cols-1]=False
	if len(iavg.shape)==3:
		iavg=cv2.cvtColor(iavg,cv2.COLOR_BGR2GRAY)
	print('done')
	print(datetime.datetime.fromtimestamp(time.time()).strftime('%Y/%m/%d %H:%M:%S'))

	print('Search max...',end='',flush=True)
	windowsize=int(16*(rows+cols)/(2448+2048))
	maxi=0
	maxy=0
	maxx=0
	for y in range(0,rows):
		for x in range(0,cols):
			if inside[y,x]:
				if iavg[y,x]>maxi:
					maxi=iavg[y,x]
					maxy=y
					maxx=x
	albedo=np.average(iavg[maxy-windowsize:maxy+windowsize,maxx-windowsize:maxx+windowsize])
	print('done')

	print('Calc normal...',end='',flush=True)
	normal=np.zeros((rows,cols,3),dtype=np.float32)
	for y in range(0,rows):
		for x in range(0,cols):
			if inside[y,x]:
				nz=iavg[y,x]/albedo
				nz=np.clip(nz,0,1)
				nxy=np.sqrt(1-nz*nz)
				nx=nxy*np.cos(aolp[y,x])
				ny=nxy*np.sin(aolp[y,x])

				ix=-(iavg[y,x+1]-iavg[y,x-1])
				iy=-(iavg[y-1,x]-iavg[y+1,x])

				if nx*ix+ny*iy<0:
					nx=-nx
					ny=-ny
			else:
				nx=0
				ny=0
				nz=1
			normal[y,x,0]=nx
			normal[y,x,1]=ny
			normal[y,x,2]=nz
	print('done')

	print(datetime.datetime.fromtimestamp(time.time()).strftime('%Y/%m/%d %H:%M:%S'))
	np.save('normal.npy',normal)
	cv2.imwrite('normal.bmp',((normal+[1,1,1])/2*255)[:,:,::-1].astype(np.uint8))

	print('Download normal.npy and normal.bmp')
	print('surface normal. xyz. -1~1. y-axis upper direction. z-axis from display to you.')
	files.download('normal.npy')
	files.download('normal.bmp')

if __name__ == '__main__':
	main()
